﻿namespace HeuristicLab.Tests.Benchmark.Problem {
  using System;
  using System.Collections.Generic;
  using System.Diagnostics;
  using System.Linq;
  using System.Threading.Tasks;

  using HeuristicLab.Algorithms.PushGP.Data.Random;
  using HeuristicLab.Algorithms.PushGP.Expressions;
  using HeuristicLab.Algorithms.PushGP.Generators;
  using HeuristicLab.Algorithms.PushGP.Interpreter;

  using Microsoft.VisualStudio.TestTools.UnitTesting;

  [TestClass]
  public class CountOdds : Problem<long[], long> {
    public CountOdds() {
      this.Examples = this.GetExamples(
        "examples-count-odds.csv",
        ExampleConverter.Integers,
        ExampleConverter.Integer);
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void RandomWalk() {
      var maxProgramSizeLimit = 100;
      var iterations = 4000;
      var best = double.MaxValue;
      var globalExecCounter = 0;
      var lockObj = new object();
      var lockCount = new object();
      RandomFactory.Seed = 1337;

      Expression bestProgram = null;
      var config = new Configuration { EvalPushLimit = 500 };
      var pool = new PushGpInterpreterPool(config);

      Parallel.For(0, iterations, i => {

        var execCounter = 0;
        var program = CodeGenerator.RandomProgram(maxProgramSizeLimit);
        var results = new List<long>();

        using (var interpreter = pool.GetInstance()) {
          foreach (var example in this.Examples) {
            if (example.Input.Length > 0) interpreter.IntegerStack.Push(example.Input);

            interpreter.Interpret(program);

            if (!interpreter.IntegerStack.IsEmpty) {
              var diff = example.Output - interpreter.IntegerStack.Top;
              var value = diff == long.MinValue ? 10000 : Math.Min(Math.Abs(diff), 10000);

              results.Add(value);
            }

            execCounter += interpreter.ExecCounter;
            interpreter.Clear();
          }
        }

        lock (lockCount) {
          globalExecCounter += execCounter;
        }

        if (results.Count == 0) return;

        var avg = results.Average();

        if (avg >= best) return;

        lock (lockObj) {
          if (avg < best) {
            best = avg;
            bestProgram = program;
          }
        }
      });

      Debug.WriteLine("Best: {0}", best);
      Debug.WriteLine("Program: {0}", bestProgram);
      Debug.WriteLine("ExecCounter: {0}", globalExecCounter);
    }
  }
}